//******************************************************************************
// The example reads the RTCC time from the MCP7940 and send it via UART.
// The mini Sensing - USB board is required to properly run the example. 
//
//
// Tested Platform: mini Controller Board PIC32
// Tested MCU: PIC32MX150F128D
// Tested Compiler: XC32 4.30
//******************************************************************************

#include "xc.h"

#define SYSTEM_CLOCK 32000000UL

#include "LTlib.h"

#include "HW_mini_Sensing_MCU_Board.h"
#include "HW_mini_Sensing_USB.h"

#include "module_IO.h"  
#include "module_IO.c" 

#include "module_UART.h"  
#include "module_UART.c" 

#include "module_I2C.h"
#include "module_I2C.c"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#define MCP7940_I2C_MODULE_2
#include "MCP7940.h"
#include "MCP7940.c"


int main (void){
    
    IO_set_all_ports_as_inputs ();
    
    UART1_open (UART_BAUDRATE_115200); 
    
    //Set LED pin as output
    P16_DIR = SET_AS_OUTPUT;
  
    //Turn OFF the LED
    P16_OUT = LED_OFF;

    delay_set_quartz(32);
    
    RTCC_initialize(400);
    RTCC_set_years(0x24);
    RTCC_set_months(0x11);
    RTCC_set_days(0x22);
    
    RTCC_set_hours(0x16);
    RTCC_set_minutes(0x20);
    RTCC_set_seconds(0x22);

    while (1){
        
        delay_ms (1000);            
        P16_OUT ^= LED_ON;        
        
        UART1_write_message ("Date: "); 
        UART1_write_string(RTCC_get_date());
        UART1_write_byte(10);
                
        UART1_write_message ("Time: "); 
        UART1_write_string(RTCC_get_time_seconds());
        UART1_write_byte(10);
        
        UART1_write_byte(10);
        
    }    
}

