//******************************************************************************
// This examples stores in an incremtal file name data from the sensor (gir. and acc.)
// It requires the USB board for the SW1 and SW2. SW1 starts the data logging.
// SW2 stops and close the file. 
// LED1 on the Memory boards blinks on the daa writing process.
// LED2 on the PIC Board is turned ON once the file is closed. 
//
// Tested Platform: mini Sensing Board PIC32
// Tested MCU: PIC32MX150F128D
// Tested Compiler: XC32 4.35
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "HW_mini_Sensing_MCU_Board.h"
#include "HW_mini_Sensing_Memory_Card.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#define DECIMAL_SEPARATOR ','
#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_SPI.h"
#include "module_SPI.c"

#include "module_I2C.h"
#include "module_I2C.c"

#define MCP7940_I2C_MODULE_2
#include "MCP7940.h"
#include "MCP7940.c"

#define MEMORY_SPI_MODULE_1
#define MEMORY_BOARD_WITH_RTC
#include "memory_raw.h"
#include "memory_raw.c"

#include "FAT.h"
#include "FAT.c"

#include "LSM6DSM.h"
#include "LSM6DSM.c"

#define DEVICE_ADDRESS 0x6A

#define STATE_MEMORY_IN 0x03
#define STATE_MEMORY_OUT 0x02
#define STATE_RUNNING 0x01
#define STATE_IDLE 0x00


int main(void) {
    
    FATFS sd_card;	//Pointer to the mounted memory
    FIL file_1;		//File pointer

    uint16_t byte_written;
   
    uint8_t temp_string[6];
    uint8_t status = 0;
    uint8_t state = STATE_MEMORY_OUT;
    
    uint8_t FILE_NAME_BASE[] = "Log_";
    uint8_t FILE_NAME_EXTENSION[] = ".txt";
    uint8_t FILE_HEADER[] = "Date   Time   Acc_X   Acc_Y   Acc_Z   Gyro_X   Gyro_Y   Gyro_Z";
    uint16_t file_counter = 0;
    uint8_t file_name[20];  
    
    float Acc_X = 0;
    float Acc_Y = 0;
    float Acc_Z = 0;
    
    float dps_X = 0;
    float dps_Y = 0;
    float dps_Z = 0;
    
    delay_set_quartz(32);
    
    IO_set_all_ports_as_inputs();
     
    LED_1_DIR = SET_AS_OUTPUT;
    LED_2_DIR = SET_AS_OUTPUT;
    LED_1_MEMORY_DIR = SET_AS_OUTPUT;
    CARD_DETECTION_SWITCH_DIR = SET_AS_INPUT;
    
    LED_1_MEMORY = LED_OFF; 
    LED_1 = LED_OFF; 
    LED_2 = LED_OFF;

    delay_ms (500);
    
    //*******************************************************
    // RTC initialization
    //*******************************************************
    
    RTCC_initialize (400);
    
    //You can use it for the first initialization
    //if the system has a battery it will keep it running.
    //Comment it the second time you program, to use the previous time. 
    RTCC_set_seconds(0x00);
    RTCC_set_minutes(0x52);     
    RTCC_set_hours(0x08);  
    
    RTCC_set_days(0x15);
    RTCC_set_months(0x05); 
    RTCC_set_years(0x24); 
    
    //*******************************************************
    // Sensor initialization
    //*******************************************************
    
    LSM6DSM_initialize (DEVICE_ADDRESS, 400);
    LSM6DSM_set_full_scale (DEVICE_ADDRESS,LSM6DSM_ACCELEROMETER, LSM6DSM_ACC_FULLSCALE_2g);
    LSM6DSM_set_sampling_frequency (DEVICE_ADDRESS, LSM6DSM_ACCELEROMETER, LSM6DSM_ACC_SAMPLING_52_HZ);
    
    LSM6DSM_set_full_scale (DEVICE_ADDRESS,LSM6DSM_GYROSCOPE, LSM6DSM_GYRO_FULLSCALE_125_dps);
    LSM6DSM_set_sampling_frequency (DEVICE_ADDRESS, LSM6DSM_GYROSCOPE, LSM6DSM_GYRO_SAMPLING_104_HZ);
    
    //*******************************************************
    // Write the file
    //*******************************************************
    
    while (1){

        if (is_memory_inserted () == MEMORY_NOT_DETECTED){            
            state = STATE_MEMORY_OUT ; 
            LED_1 = LED_ON;
            LED_2 = LED_OFF;
            
            blink_memory_LED (50);
        }
        
        
        if (state == STATE_MEMORY_OUT){
            //Wait for the card to be inserted
            while (is_memory_inserted () == MEMORY_NOT_DETECTED);
            
            blink_memory_LED (50);
            disk_initialize (status);
            
            state = STATE_MEMORY_IN;
            LED_1 = LED_OFF;
        }
        

        //Check for start condition
        if (state == STATE_MEMORY_IN){

            if (P4_IN == 0x00){
                if (f_mount(&sd_card, "", 1) == FR_OK){
                   state = STATE_RUNNING;  
                } else {
                   state = STATE_MEMORY_IN; 
                   delay_ms(100);
                }                
            }    
        }


        if (state == STATE_RUNNING){

            file_counter++;

            LED_1_MEMORY = LED_OFF; 
            LED_1 = LED_OFF; 
            LED_2 = LED_OFF;

            //Make the file name 
            LTlib_int16_to_string (temp_string, file_counter);
            LTlib_merge_strings (FILE_NAME_BASE, temp_string, file_name);
            LTlib_merge_strings (file_name, FILE_NAME_EXTENSION, file_name);

            if (f_open(&file_1, file_name, FA_CREATE_ALWAYS | FA_READ | FA_WRITE) == FR_OK){

                f_write(&file_1, FILE_HEADER, LTlib_string_length (FILE_HEADER), &byte_written);
                f_write(&file_1, &ASCII_New_Line, 1, &byte_written);  

                while (1){

                    if (is_memory_inserted () == MEMORY_DETECTED){
                        blink_memory_LED (2);
                    }

                    delay_ms(50);

                    LSM6DSM_get_interrupt_status (DEVICE_ADDRESS, &status);

                    //Write Date and Time
                    f_write(&file_1, RTCC_get_date(), 8, &byte_written);
                    f_write(&file_1, "   ", 1, &byte_written);

                    f_write(&file_1, RTCC_get_time_seconds(), 8, &byte_written);
                    f_write(&file_1, "   ", 1, &byte_written);

                    //Wait for acceleration data to be ready
                    while ((status & LSM6DSM_STATUS_XLDA) == 0){                    
                        LSM6DSM_get_interrupt_status (DEVICE_ADDRESS, &status);
                    }

                     //Wait for gyroscope data to be ready
                    while ((status & LSM6DSM_STATUS_GDA) == 0){                    
                        LSM6DSM_get_interrupt_status (DEVICE_ADDRESS, &status);
                    }

                    //Read acceleration data
                    LSM6DSM_get_acceleration(DEVICE_ADDRESS, &Acc_X, &Acc_Y, &Acc_Z);

                    LTlib_float_to_string (temp_string, Acc_X, 3 );
                    f_write(&file_1, temp_string, 6, &byte_written);
                    f_write(&file_1, "   ", 1, &byte_written);

                    LTlib_float_to_string (temp_string, Acc_Y, 3 );
                    f_write(&file_1, temp_string, 6, &byte_written);
                    f_write(&file_1, "   ", 1, &byte_written);

                    LTlib_float_to_string (temp_string, Acc_Z, 3 );
                    f_write(&file_1, temp_string, 6, &byte_written);
                    f_write(&file_1, "   ", 1, &byte_written);

                    //Read gyroscope data
                    LSM6DSM_get_angular_speed(DEVICE_ADDRESS, &dps_X, &dps_Y, &dps_Z);

                    LTlib_float_to_string (temp_string, dps_X, 3 );
                    f_write(&file_1, temp_string, 6, &byte_written);
                    f_write(&file_1, "   ", 1, &byte_written);

                    LTlib_float_to_string (temp_string, dps_Y, 3 );
                    f_write(&file_1, temp_string, 6, &byte_written);
                    f_write(&file_1, "   ", 1, &byte_written);

                    LTlib_float_to_string (temp_string, dps_Z, 3 );
                    f_write(&file_1, temp_string, 6, &byte_written);
                    f_write(&file_1, "   ", 1, &byte_written);

                    f_write(&file_1, &ASCII_New_Line, 1, &byte_written);

                    //Check for stop condition
                    if (P3_IN == 0x00){
                        f_close(&file_1); 
                        state = STATE_MEMORY_IN;
                        LED_2 = LED_ON;
                        break;     
                    }
                    
                    //Check if the memory is still there
                    if (is_memory_inserted () == MEMORY_NOT_DETECTED){ 
                        f_close(&file_1);
                        state = STATE_MEMORY_OUT ; 
                        LED_1 = LED_ON;
                        LED_2 = LED_OFF;
                        break;
                    }
                }    
                
            } else {
                //File could not be opened
                LED_1 = LED_ON;         
            } 
        
        }
    }
   
}   

    
